import asyncio
import logging
from html import parser as html_parser

from httpx import AsyncClient, URL
from pydantic import BaseModel

from knoc.settings import settings
from knoc.kmiweo_customer_info import parse_customer_info

logger = logging.getLogger("kniweo")

url_search_customer = "https://knet.miweo.com/Ajax/controlerAjaxModule.php"


class MiweoApiError(Exception):
    pass


class MiweoCustomerNotFoundError(MiweoApiError):
    pass


class MiweoIdRecord(BaseModel):
    id: str
    link: str


class MiweoRecordHtmlParser(html_parser.HTMLParser):
    record_id: MiweoIdRecord

    def handle_starttag(self, tag, attrs):
        if tag == "a":
            for attr in attrs:
                if attr[0] == "href" and isinstance(attr[1], str):
                    self.record_id = MiweoIdRecord(**URL(attr[1]).params)
                    return


class MiweoAPI(AsyncClient):
    _miweo_domain: URL
    _miweao_record_html_parser: MiweoRecordHtmlParser

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cookies.set("miweoCookie", settings.miweo.cookie)
        self.cookies.set("PHPSESSID", "md7lfnqqqk4d05kasom2874trf")
        self._miweao_record_html_parser = MiweoRecordHtmlParser()

    async def get_customer_info(self, customer_id: str):
        """Get customer info for the customer id"""
        customer_record_id = await self.get_customer_record_id(customer_id=customer_id)
        return await self.get_customer_info_from_miweo_id(customer_record_id)

    async def get_customer_record_id(self, customer_id: str) -> MiweoIdRecord:
        """Get internal Miweo id record id for the customer id"""
        resp = await self.get(
            "Ajax/controlerAjaxModule.php",
            params={
                "search": customer_id,
            },
        )
        if not resp.is_success:
            raise MiweoApiError(f"Miweo API error {resp.status_code}")
        resp_json = resp.json()
        try:
            results = resp_json["data"]["results"]
        except KeyError:
            raise MiweoApiError("Malformed Miweo response (no data.results")
        if len(results) == 0:
            raise MiweoApiError(f"No customer found with customer_id {customer_id}")
        return self.get_miweo_record_id(results[0])

    async def get_customer_info_from_miweo_id(self, customer_record_id: MiweoIdRecord):
        # https://knet.miweo.com/index.php?link=35753&id=104157&ajax=t
        resp = await self.get(
            "index.php",
            params={
                "link": customer_record_id.link,
                "id": customer_record_id.id,
                "ajax": "t",
            },
        )
        parse_customer_info(resp.text)
        breakpoint()
        ...

    def get_miweo_record_id(self, record) -> MiweoIdRecord:
        """Extract link and id for the record"""
        url = record[-1]
        self._miweao_record_html_parser.feed(url)
        if not hasattr(self._miweao_record_html_parser, "record_id"):
            raise MiweoApiError("No internal id found in record")
        return self._miweao_record_html_parser.record_id


miweo_api = MiweoAPI(
    base_url=URL(str(settings.miweo.url)),
    params={
        "type": "KwaooCrm",
        "request": "getKnetCustomerListList",
        "link": "35747",
        "knetStatusFilter": "",
        "knetOfferFilter": "",
        "knetDspFilter": "",
        "itemNumberByPage": "1",
        "page": "1",
        "order": "lastUpdate",
        "typeOrder": "desc",
    },
)


async def main(customer_id: str = "KNGP-0104157"):
    try:
        info = await miweo_api.get_customer_info(customer_id=customer_id)
    except MiweoApiError as err:
        logger.warning(err.args[0])
    else:
        logger.info(info)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    asyncio.run(main())
