from typing import Type, Tuple
from pathlib import Path
from xdg import BaseDirectory

from pydantic import BaseModel, HttpUrl
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    YamlConfigSettingsSource,
)
from hatchet_sdk import ClientConfig as HatchetClientConfig

config_files = [
    Path(BaseDirectory.xdg_config_home) / "knoc" / "conf.yaml",
    Path(Path.cwd().root) / "etc" / "knoc" / f"conf.yml",
]


class KSplynxSettings(BaseModel):
    url: HttpUrl = HttpUrl("https://gp-acs.k-net.fr")
    key: str = ""
    sec: str = ""
    authtypeapikey: str = "api_key"


class HatchetSettings(BaseModel):
    token: str = ""
    tlsstrategy: str = "none"


class KMiweoSettings(BaseModel):
    url: HttpUrl = HttpUrl("https://knet.miweo.com")
    cookie: str = ""


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        nested_model_default_partial_update=True,
        env_nested_delimiter="_",
        env_file=(".env", ".env.prod"),
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        configs = [
            YamlConfigSettingsSource(settings_cls, yaml_file=cf) for cf in config_files
        ]
        return (
            env_settings,
            init_settings,
            dotenv_settings,
            file_secret_settings,
            *configs,
        )

    miweo: KMiweoSettings = KMiweoSettings()
    splynx: KSplynxSettings = KSplynxSettings()
    hatchet_client: HatchetClientConfig = HatchetClientConfig()

    data_path: Path = Path("data")


settings = Settings()
